"use strict";

var md5 = require('js-md5');

var localData = {
  data: {},
  setItem: function setItem(key, value) {
    this.data[key] = value;
  },
  clear: function clear() {
    this.data = {};
  },
  getItem: function getItem(key) {
    return this.data[key];
  },
  removeItem: function removeItem(key) {
    this.data[key] = null;
  }
};

var getStorage = function getStorage() {
  if (!window.localStorage) {
    return localData;
  } else {
    return window.localStorage;
  }
};

var TimeUnit = {
  SECONDS: 'SECONDS',
  MINUTES: 'MINUTES',
  HOURS: 'HOURS',
  DAYS: 'DAYS'
};
var LocalTimeUnit = {
  SECONDS: 1000,
  MINUTES: 1000 * 60,
  HOURS: 1000 * 60 * 60,
  DAYS: 1000 * 60 * 60 * 24
};

var removeItem = function removeItem(key) {
  if (typeof key !== 'string') {
    key = JSON.stringify(key);
  }

  getStorage().removeItem(md5(key));
};

var saveStartTime = function saveStartTime(startTime) {
  if (getStartTime() > parseInt("".concat(startTime || '0'))) {
    return;
  }

  getStorage().setItem('startTime', startTime);
};

var getStartTime = function getStartTime() {
  return parseInt(getStorage().getItem('startTime') || '0');
};

var saveData = function saveData(config) {
  if (config && config.key && config.url && config.data) {
    var url = config.url;
    var key = config.key;
    var cache = {
      url: url,
      key: key,
      persistent: config.persistent,
      startTime: getStartTime(),
      expires: !config.persistent ? new Date().getTime() + (config.expires || 3) * LocalTimeUnit[config.timeUnit || 'SECONDS'] : 0,
      data: config.data
    };
    getStorage().setItem(key, JSON.stringify(cache));
  }
};

var expires = function expires() {
  var storage = getStorage();

  for (var key in storage) {
    var item = storage.getItem(key);

    if (!item) {
      continue;
    }

    var cache = JSON.parse(item);

    if (!cache.persistent && cache.expires < new Date().getTime()) {
      getStorage().removeItem(key);
    }
  }
};

var getData = function getData(key, callback) {
  var cache = JSON.parse(getStorage().getItem(key));

  if (cache) {
    if (cache.startTime < getStartTime()) {
      getStorage().removeItem(key);
      return;
    }

    if (cache.persistent || cache.expires >= new Date().getTime()) {
      return new Promise(function (resolve, reject) {
        if (callback) {
          callback(cache.data);
        }

        resolve(cache.data);
      });
    }

    getStorage().removeItem(key);
  }
};

module.exports = function () {
  setInterval(expires, LocalTimeUnit.MINUTES);
  return {
    saveData: saveData,
    getData: getData,
    saveStartTime: saveStartTime,
    removeItem: removeItem,
    LocalTimeUnit: LocalTimeUnit,
    TimeUnit: TimeUnit
  };
};