"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = exports.TimeUnit = exports.LocalTimeUnit = exports.removeCacheItem = exports.saveData = exports.getCacheData = void 0;

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

var axios = require('axios');

var md5 = require('js-md5');

var _require = require('./cache')(),
    saveData = _require.saveData,
    saveStartTime = _require.saveStartTime,
    getData = _require.getData,
    removeItem = _require.removeItem,
    TimeUnit = _require.TimeUnit,
    LocalTimeUnit = _require.LocalTimeUnit;

exports.LocalTimeUnit = LocalTimeUnit;
exports.TimeUnit = TimeUnit;
exports.saveData = saveData;
var getCacheData = getData;
exports.getCacheData = getCacheData;
var saveCacheData = saveData;
var removeCacheItem = removeItem;
exports.removeCacheItem = removeCacheItem;
var timeoutUrl = {};

var saveCacheByUrl = function saveCacheByUrl(value, data, urls) {
  var url = value.url || value;
  var config = urls[url] || {};

  if (config) {
    config.url = url;
    config.key = md5(JSON.stringify(value));
    config.data = {
      data: data
    };
    saveData(config);
  }
};

var getCacheByUrl = function getCacheByUrl(value, urls) {
  var config = urls[value.url] || {};

  if (config) {
    var data = getData(md5(JSON.stringify(value)));
    return data;
  }
};

var requestByCache = function requestByCache(_ref) {
  var config = _ref.config,
      request = _ref.request,
      urls = _ref.urls;
  var md = md5(JSON.stringify(config));

  if (timeoutUrl[md]) {
    if (!timeoutUrl[md].data) {
      return new Promise(function (resolve, reject) {
        reject({
          code: '1003',
          message: '正在加载数据，请等待！！'
        });
      });
    } else {
      delete timeoutUrl[md];
    }
  }

  timeoutUrl[md] = {};
  return (getCacheByUrl(config, urls) || request(config).then(function (response) {
    saveStartTime(response.headers['start-time']);
    saveCacheByUrl(config, response.data, urls);
    return response;
  })).then(function (response) {
    timeoutUrl[md].data = response.data;
    return response;
  }, function (error) {
    delete timeoutUrl[md];
    return error;
  })["catch"](function () {
    return delete timeoutUrl[md];
  });
};

var _default = function _default(_ref2) {
  var baseURL = _ref2.baseURL,
      _ref2$timeout = _ref2.timeout,
      timeout = _ref2$timeout === void 0 ? 5000 : _ref2$timeout,
      _ref2$tokenKey = _ref2.tokenKey,
      tokenKey = _ref2$tokenKey === void 0 ? 'Authorization' : _ref2$tokenKey,
      cacheUrls = _ref2.cacheUrls,
      _ref2$requestIntercep = _ref2.requestInterceptor,
      requestInterceptor = _ref2$requestIntercep === void 0 ? function () {} : _ref2$requestIntercep,
      _ref2$getToken = _ref2.getToken,
      getToken = _ref2$getToken === void 0 ? function () {} : _ref2$getToken;
  var request = axios.create({
    baseURL: baseURL,
    timeout: timeout
  });
  request.interceptors.request.use(function (_ref3) {
    var config = _extends({}, _ref3);

    requestInterceptor(config);
    config.headers[tokenKey] = getToken();
    return config;
  }, function (error) {
    throw error;
  });
  request.interceptors.response.use(function (response) {
    return response;
  }, function (error) {
    var rejectObject = {
      error: error,
      type: 'error',
      message: '未定义的错误。'
    };

    if (!error.response) {
      rejectObject.message = '网络异常，请稍后刷新重试！';
      rejectObject.code = '1001';
    } else if (error.response.status === 401 || error.response.status === 403) {
      rejectObject.message = '无权访问！';
      rejectObject.code = '1002';
      rejectObject.type = 'low.power';
    } else {
      if (error.response.status === 502) {
        rejectObject.message = '网络繁忙，请稍候重试！';
      } else if (error.response && error.response.data && error.response.data.message) {
        rejectObject.message = error.response.data.message;
      } else if (error.message) {
        rejectObject.message = error.message;
      }
    }

    return Promise.reject(rejectObject);
  });
  return {
    request: request,
    finds: function finds(config) {
      if (cacheUrls) {
        return requestByCache({
          config: _objectSpread(_objectSpread({}, config), {}, {
            method: 'get'
          }),
          request: request,
          urls: cacheUrls
        });
      } else {
        return request(config);
      }
    },
    post: function post(config) {
      request(_objectSpread(_objectSpread({}, config), {}, {
        method: 'post'
      }));
    },
    put: function put(config) {
      request(_objectSpread(_objectSpread({}, config), {}, {
        method: 'put'
      }));
    },
    del: function del(config) {
      request(_objectSpread(_objectSpread({}, config), {}, {
        method: 'delete'
      }));
    },
    get: function get(config) {
      request(_objectSpread(_objectSpread({}, config), {}, {
        method: 'get'
      }));
    }
  };
};

exports["default"] = _default;